#!/bin/bash
# Program:
#        Install Supermicro Thin-agent
# History:
# 20140123 billw
PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin:~/bin
export PATH
prog="Thin-Agent Service"
directory="/etc/supermicro"
typeset ret_code
ret=""
os="other"
force=$1

grep "debian" /etc/issue -i -q
if [ $? == 0 ];then
os="debian"
fi

grep "ubuntu" /etc/issue -i -q
if [ $? == 0 ];then
os="ubuntu"
fi

grep "suse" /etc/issue -i -q
if [ $? == 0 ];then
os="suse"
fi

#echo $os

ipmi_msghandler_missing() {
    echo "Module ipmi_msghandler not found."
}

ipmi_devintf_missing() {
    echo "Module ipmi_devintf not found."
}

ipmi_si_missing() {
    echo "Module ipmi_si not found."
}

ethtoolFail() {
    echo "install ethtool....."
    if [ "$os" == "other" ]; then
      yum install ethtool -y
      ret=$(rpm -q ethtool)
      ret_code=$?
    elif [ "$os" == "suse" ]; then
      zypper install ethtool -y
      ret=$(rpm -q ethtool)
      ret_code=$?
    else
      apt-get install ethtool -y
      ret=$(dpkg-query -W ethtool)
      ret_code=$?
   fi
   [ $ret_code == 0 ] || fail
}

mdadmFail() {
    #echo "install mdadm....."
    while true; do
        if [ "$force" == "-y" ]; then
	    yn="Y"
	else
            read -p "To enable Intel RSTe information, please install mdadm package [y/n] " yn
	fi

	case $yn in
       	    [Yy]* )    
				if [ "$os" == "other" ]; then
					yum install mdadm -y
					ret=$(rpm -q mdadm)
					ret_code=$?
                                elif [ "$os" == "suse" ]; then
					zypper install mdadm -y
      					ret=$(rpm -q mdadm)
					ret_code=$?
				else
					apt-get install mdadm -y
					ret=$(dpkg-query -W mdadm)
					ret_code=$?
				fi

                [ $ret_code == 0 ] || fail
                break;;
            [Nn]* ) break;;
            * ) echo "Please answer yes or no.";;
        esac
    done
}

routeFail() {
    echo "install net-tools....."
    if [ "$os" == "other" ]; then
      yum install net-tools -y
      ret=$(rpm -q net-tools)
      ret_code=$?
    elif [ "$os" == "suse" ]; then
      zypper install net-tools -y
      ret=$(rpm -q net-tools)
      ret_code=$?
   else
      apt-get install net-tools -y
      ret=$(dpkg-query -W net-tools)
      ret_code=$?
   fi
   [ $ret_code == 0 ] || fail

}

lsscsiFail() {
    echo "install lsscsi....."
    if [ "$os" == "other" ]; then
      yum install lsscsi -y
      ret=$(rpm -q lsscsi)
      ret_code=$?
    elif [ "$os" == "suse" ]; then
      zypper install lsscsi -y
      ret=$(rpm -q lsscsi)
      ret_code=$?
   else
      apt-get install lsscsi -y
      ret=$(dpkg-query -W lsscsi)
      ret_code=$?
   fi
   [ $ret_code == 0 ] || fail

}

bcFail() {
    echo "install bc....."
    if [ "$os" == "other" ]; then
      yum install bc -y
      ret=$(rpm -q bc)
      ret_code=$?
    elif [ "$os" == "suse" ]; then
      zypper install bc -y
      ret=$(rpm -q bc)
      ret_code=$?
   else
      apt-get install bc -y
      ret=$(dpkg-query -W bc)
      ret_code=$?
   fi
   [ $ret_code == 0 ] || fail

}


smarttoolFail() {
    #echo "install smartmontools....."
    while true; do
        if [ "$force" == "-y" ]; then
	    yn="Y"
	else
            read -p "To enable SMART HDD information, please install smartmontools package [y/n] " yn
	fi

	case $yn in
       	    [Yy]* )    
	        if [ "$os" == "other" ]; then
         	    yum install smartmontools* -y
	            ret=$(rpm -q smartmontools)
        	    ret_code=$?
		elif [ "$os" == "suse" ]; then
      		    zypper install smartmontools* -y
      		    ret=$(rpm -q smartmontools)
		    ret_code=$?
        	else
           	    apt-get install smartmontools* -y
                    ret=$(dpkg-query -W smartmontools)
                    ret_code=$?
                fi

                [ $ret_code == 0 ] || fail
                break;;
            [Nn]* ) break;;
            * ) echo "Please answer yes or no.";;
        esac
    done
}

fail() {
    echo "install fail"
    exit 1
}

IPMITAS_missing() {
    echo "IPMITAS not found"
    fail
}

tas_missing() {
    echo "tas not found"
    fail
}

watchtas_missing() {
    echo "watchtas not found"
    fail
}

storcli_missing() {
    echo "storcli not found"
}

copyFile() {
    cp -rf IPMITAS "$directory"
    cp -rf tas /etc/init.d/
    cp -rf watchtas "$directory"
    cp -rf storcli /usr/bin
	cp -rf license.txt "$directory"
    echo ""
    echo "*****StorCLI License start*****"
    cat $directory/license.txt
    echo "*****StorCLI License end*****"
}

checkIPMI() {
    ret_code=1

    if [ "$os" == "other" ];then
        ret=$(service ipmi status | grep ipmi_msghandler)
        ret_code=$?
    fi

    if [ $ret_code == 1 ]; then
        ret=$(modprobe ipmi_msghandler)
        ret_code=$?
        [ $ret_code == 0 ] || ipmi_msghandler_missing
    fi

    ret_code=1

    if [ "$os" == "other" ];then
        ret=$(service ipmi status | grep ipmi_devintf)
        ret_code=$?
    fi

    if [ $ret_code == 1 ]; then
    	ret=$(modprobe ipmi_devintf)
	ret_code=$?
	[ $ret_code == 0 ] || ipmi_devintf_missing
    fi

    ret_code=1

    if [ "$os" == "other" ];then
        ret=$(service ipmi status | grep ipmi_si)
        ret_code=$?
    fi

    if [ $ret_code == 1 ]; then
	ret=$(modprobe ipmi_si)
	ret_code=$?
	[ $ret_code == 0 ] || ipmi_si_missing
    fi
}

checkTool() {
   if [ "$os" == "other" ] || [ "$os" == "suse" ]; then
      ret=$(rpm -q ethtool)
      ret_code=$?
      [ $ret_code == 0 ] || ethtoolFail
      ret=$(rpm -q smartmontools)
      ret_code=$?
      [ $ret_code == 0 ] || smarttoolFail
      ret=$(rpm -q mdadm)
      ret_code=$?
      [ $ret_code == 0 ] || mdadmFail
      ret=$(rpm -q net-tools)
      ret_code=$?
      [ $ret_code == 0 ] || routeFail
      ret=$(rpm -q lsscsi)
      ret_code=$?
      [ $ret_code == 0 ] || lsscsiFail
      ret=$(rpm -q bc)
      ret_code=$?
      [ $ret_code == 0 ] || bcFail	
   else
      ret=$(dpkg-query -l ethtool | grep ethtool | awk '{print $1}')
      ret_code=$?
      [ $ret_code == 0 ] || ethtoolFail
      [ "$ret" == "ii" ] || ethtoolFail
      ret=$(dpkg-query -l smartmontools | grep smartmontools | awk '{print $1}')
      ret_code=$?
      [ $ret_code == 0 ] || smarttoolFail
      [ "$ret" == "ii" ] || smarttoolFail
      ret=$(dpkg-query -l mdadm | grep mdadm | awk '{print $1}')
      ret_code=$?
      [ $ret_code == 0 ] || mdadmFail
      [ "$ret" == "ii" ] || mdadmFail
      ret=$(dpkg-query -l net-tools | grep net-tools | awk '{print $1}')
      ret_code=$?
      [ $ret_code == 0 ] || routeFail
      [ "$ret" == "ii" ] || routeFail
      ret=$(dpkg-query -l lsscsi | grep lsscsi | awk '{print $1}')
      ret_code=$?
      [ $ret_code == 0 ] || lsscsiFail
      [ "$ret" == "ii" ] || lsscsiFail	
      ret=$(dpkg-query -l bc | grep bc | awk '{print $1}')
      ret_code=$?
      [ $ret_code == 0 ] || bcFail
      [ "$ret" == "ii" ] || bcFail
   fi   
}

check() {
    [ -f IPMITAS ] || IPMITAS_missing
    [ -f tas ] || tas_missing
    [ -f watchtas ] || watchtas_missing
    [ -f storcli ] || storcli_missing

    if [ -x IPMITAS ]; then
        if [ -x tas ]; then
            if [ -x watchtas ]; then
                checkTool
                checkIPMI
            else
                echo "watchtas not executable"
                fail
            fi   
        else
            echo "tas not executable"
            fail
        fi       
    else
       echo "IPMITAS not executable"
       fail
    fi
}

startup() {
    grep "SUSE" /etc/issue -i -q
    
    if [ $? -eq 0 ];then
        ln -f -s /etc/init.d/tas /etc/init.d/rc0.d/S99tas
        ln -f -s /etc/init.d/tas /etc/init.d/rc1.d/S99tas
        ln -f -s /etc/init.d/tas /etc/init.d/rc2.d/S99tas
        ln -f -s /etc/init.d/tas /etc/init.d/rc3.d/S99tas
        ln -f -s /etc/init.d/tas /etc/init.d/rc4.d/S99tas
        ln -f -s /etc/init.d/tas /etc/init.d/rc5.d/S99tas
        ln -f -s /etc/init.d/tas /etc/init.d/rc6.d/S99tas
    else
        ln -f -s /etc/init.d/tas /etc/rc0.d/S99tas
        ln -f -s /etc/init.d/tas /etc/rc1.d/S99tas
        ln -f -s /etc/init.d/tas /etc/rc2.d/S99tas
        ln -f -s /etc/init.d/tas /etc/rc3.d/S99tas
        ln -f -s /etc/init.d/tas /etc/rc4.d/S99tas
        ln -f -s /etc/init.d/tas /etc/rc5.d/S99tas
        ln -f -s /etc/init.d/tas /etc/rc6.d/S99tas
    fi

    if [ "$os" == "other" ] || [ "$os" == "suse" ]; then
        chkconfig tas on
    else
        update-rc.d -f tas remove
        update-rc.d tas defaults
    fi
}

echo "Install" $prog
check

if [ -d "$directory" ]
then
   copyFile
else
    mkdir "$directory"
    copyFile
fi

startup
echo "Install success..."
killall IPMITAS 2> /dev/null

if [ $? != 0 ]; then
    pkill -9 -x IPMITAS
fi

killall watchtas 2> /dev/null

if [ $? != 0 ]; then
    pkill -9 -x watchtas
fi

/etc/init.d/tas start

